// TruPhysics engine v1.0 - Miniature Physics Engine by MrKeen
// Mainly used for vehicles but can be expanded.

// truPhysics init()

$truPhysics::enabled = true;                  // Enable the physics engine?
$truPhysics::debugMode = false;               // Enable debug messages?
$truPhysics::maxVelocity = 1500;              // 1500 m/s or about 5400 KPH, T2's universal speed limit

function truPhysics::onInit(%this)
{
   truPhysics.enabled = $truPhysics::enabled;
   truPhysics.debugMode = $truPhysics::debugMode;

   truPhysics.Version = "1.9.1.1";
   truPhysics.maxVelocity = $truPhysics::maxVelocity;
   
   ActivatePackage(dynCollDamageCalc);
}

function truPhysics::onDie(%this)
{
   DeactivatePackage(dynCollDamageCalc);
}

//if(!SuperFist && !truPhysics) // for base servers
if(!isObject(SuperFist) && !isObject(truPhysics)) // for base servers --- (ST)
{
   new ScriptObject(truPhysics)
   {
      class = truPhysics;
   };

   truPhysics.enabled = $truPhysics::enabled;
   truPhysics.debugMode = $truPhysics::debugMode;

   truPhysics.Version = "1.9.1.1";
   truPhysics.maxVelocity = $truPhysics::maxVelocity;
   
   ActivatePackage(dynCollDamageCalc);
}
else // for SuperFist(tm) enabled servers
   //if(!truPhysics) --- (ST)
   if(!isObject(truPhysics))
      SuperFist.addClass(truPhysics);

//------------------------------------------------------------------------------
// Let the physics BEGIN!
//------------------------------------------------------------------------------

function calculateAcceleration(%obj)
{
   if(isObject(%obj))
   {
      if(%obj.calcAccelThread)
         cancel(%obj.calcAccelThread);

      %vel = velToSingle(%obj.getVelocity());

      if(%obj.lastVel $= "")
      {
         %obj.lastVel = %vel;
         %obj.accel = 0;
      }
   
      if(%vel > 1)
         %accel = mAbs(%vel - %obj.lastVel); // a = |dv/dt = vf - vi/tf - ti| (so we dont throw vehicles in our direction upon collision)
      else
         %accel = 0;
      
      %obj.lastVel = %vel;
      %obj.accel = %accel;

      return %accel;
   }
}

//------------------------------------------------------------------------------
// vehicleTick() functions
//------------------------------------------------------------------------------

function HoverVehicleData::create(%block, %team, %oldObj)
{
   if(%oldObj $= "")
   {
      %obj = new HoverVehicle()
      {
         dataBlock  = %block;
         respawn    = "0";
         teamBought = %team;
         team = %team;
      };
   }
   else
   {
      %obj = new HoverVehicle()
      {
         dataBlock  = %data;
         respawn    = "0";
         teamBought = %team;
         team = %team;
         mountable = %oldObj.mountable;
         disableMove = %oldObj.disableMove;
         resetPos = %oldObj.resetPos;
         respawnTime = %oldObj.respawnTime;
         marker = %oldObj;
      };
   }

   schedule(32, 0, vehicleTick, %obj);
   return(%obj);
}

function WheeledVehicleData::create(%data, %team, %oldObj)
{
   if(%oldObj $= "")
   {
      %obj = new WheeledVehicle()
      {
         dataBlock  = %data;
         respawn    = "0";
         teamBought = %team;
         team = %team;
      };
   }
   else
   {
      %obj = new WheeledVehicle()
      {
         dataBlock  = %data;
         respawn    = "0";
         teamBought = %team;
         team = %team;
         mountable = %oldObj.mountable;
         disableMove = %oldObj.disableMove;
         resetPos = %oldObj.resetPos;
         deployed = %oldObj.deployed;
         respawnTime = %oldObj.respawnTime;
         marker = %oldObj;
      };
   }

   schedule(32, 0, vehicleTick, %obj);
   return(%obj);
}

function FlyingVehicleData::create(%data, %team, %oldObj)
{
   if(%oldObj $= "")
   {
      %obj = new FlyingVehicle()
      {
         dataBlock  = %data;
         respawn    = "0";
         teamBought = %team;
         team = %team;
      };
   }
   else
   {
      %obj = new FlyingVehicle()
      {
         dataBlock  = %data;
         teamBought = %team;
         team = %team;
         mountable = %oldObj.mountable;
         disableMove = %oldObj.disableMove;
         resetPos = %oldObj.resetPos;
         respawnTime = %oldObj.respawnTime;
         marker = %oldObj;
      };
   }

   schedule(32, 0, vehicleTick, %obj);
   return(%obj);
}

function vehicleTick(%obj)
{
   if(isObject(%obj))
   {
      calculateAcceleration(%obj);
      %obj.calcAccelThread = schedule(32, 0, vehicleTick, %obj);
   }
}

//------------------------------------------------------------------------------

function capVelocity(%vel)
{
   if(velToSingle(%vel) > truPhysics.maxVelocity)
      return vectorFloor(vectorScale(vectorNormalize(%vel), (1 - truPhysics.maxVelocity)));
   else
      return %vel;
}

//------------------------------------------------------------------------------
// onCollision() - oh YEAH baby! Lets see some friction!
//------------------------------------------------------------------------------

function VehicleData::onCollision(%data, %obj, %col)
{
   if(truPhysics.debugMode)
      error("onCollision: "@%data.getClassName() SPC %obj.getClassName() SPC %col.getClassName());

//   if(%data.disablePhysics || !truPhysics.enabled) // !enabled = default vehicle physics, local or global scope
//      return;

   %accel = calculateAcceleration(%obj); 

   if(%col.isVehicle())  // SMAAAAAAAAAAAAAAAAAAAAAACK!
   {
      if(%col.getDamageState() $= Destroyed)  // prevent collisions with non-existant objects
         return;

      if(%accel > 1)
      {
         %mv = %obj.getVelocity();
         %myvel = velToSingle(%mv);
         %myvec = vectorNormalize(%mv);
         %mymass = %obj.getMass();
         %force = vectorScale(%myvec, (%mymass*%accel));

         if(truPhysics.debugMode)
            error(%col SPC %col.getClassName() SPC %force);

         %col.applyImpulse(%col.getPosition(), %force); // pang!!

         if(%col.team != %obj.team)
         {
            %col.truPhysicsHandled = true;
            %forceDmg = ((%mymass * 0.01) + (%myvel * 0.01)) * 0.561;
            %col.getDatablock().damageObject(%col, %obj, "0 0 0", %forceDmg, $DamageType::Impact, "0 0 0");
         }
      }
   }
//   else if(%col.isPlayer())  // ow!
//   {
//      %mv = %obj.getVelocity();
//      %myvec = vectorNormalize(%mv);
//      %mymass = %obj.getDatablock().mass;
//      %force = vectorScale(%myvec, (%mymass*%accel));
//
//      %col.applyImpulse(%col.getPosition(), %force); // pang!!
//
//      if(%col.client.team != %obj.team)
//      {
//         // damage occours
//      }
//   }
}

function xVehicleData::onCollision(%data, %obj, %col)
{
   if(truPhysics.debugMode)
      error("onCollision: "@%data.getClassName() SPC %obj.getClassName() SPC %col.getClassName());

//   if(%data.disablePhysics || !truPhysics.enabled) // !enabled = default vehicle physics, local or global scope
//      return;

   if(%col.isVehicle())  // ow!
   {
      if(%col.getDamageState() $= Destroyed)  // prevent collisions with non-existant objects
         return;

      %mv = %obj.getVelocity();
      %cv = %col.getVelocity();

      %myvel = velToSingle(%mv);
      %itsvel = velToSingle(%cv);

      %accel = calculateAcceleration(%obj); //%myvel - %itsvel;

      if(%accel > 1)
      {
         %myvec = vectorNormalize(%mv);
         %myvel = msToKPH(%myvel);
         %mymass = %obj.getDatablock().mass;

         %force = vectorScale(%myvec, ((%myvel*5)+(%mymass*5))*2.5);
//         %colPos = tpGetFacingObject(%obj, 10);

         if(truPhysics.debugMode)
            error(%col SPC %col.getClassName() SPC %colpos SPC %force);

         %col.applyImpulse(%col.getPosition(), %force); // pang!!
      }
   }

//   Parent::onCollision(%data, %obj, %col);
}

package dynCollDamageCalc {

function VehicleData::damageObject(%data, %targetObject, %sourceObject, %position, %amount, %damageType, %momVec, %theClient, %proj)
{
   if(%damageType == $DamageType::Impact && %targetObject.truPhysicsHandled) //handled by truPhysics
      %amount = 0;
   else
      %targetObject.truPhysicsHandled = false;

   Parent::damageObject(%data, %targetObject, %sourceObject, %position, %amount, %damageType, %momVec, %theClient, %proj);
}
};

function tpGetFacingObject(%obj, %dist)
{
   %vec = %obj.getForwardVector();
   %pos = %obj.getWorldBoxCenter();
   %nVec = VectorNormalize(%vec);
   %scVec = VectorScale(%nVec, %dist);
   %end = VectorAdd(%pos, %scVec);
   %searchResult = containerRayCast(%pos, %end, $TypeMasks::VehicleObjectType, %obj);

   if(%searchResult)
      return getWord(%searchResult, 0);
   else
      return false;
}

//------------------------------------------------------------------------------
// onImpact() - Modified for Maximum Pleasure
//------------------------------------------------------------------------------

function VehicleData::onImpact(%data, %vehicleObject, %collidedObject, %vec, %vecLen)
{
   if(%data.useDatablockColValues)
   {
      if(%collidedObject.isVehicle())
      {
         if(%vecLen > %data.minImpactSpeed)
            %data.damageObject(%vehicleObject, 0, VectorAdd(%vec, %vehicleObject.getPosition()),
                               (%vecLen * %data.speedDamageScale), $DamageType::Ground);
      }
      else
      {
         if(%vecLen > %data.minImpactSpeed)
            %data.damageObject(%vehicleObject, 0, VectorAdd(%vec, %vehicleObject.getPosition()),
                               (%vecLen * %data.speedDamageScale), $DamageType::Ground);
      }
   }
   else
   {
      if(%collidedObject.isVehicle())
      {
         if(%vecLen > %data.minImpactSpeed)
            %data.damageObject(%vehicleObject, 0, VectorAdd(%vec, %vehicleObject.getPosition()),
                               ((%vecLen * %data.speedDamageScale) / 20), $DamageType::Ground);
      }
      else
      {
         if(%vecLen > %data.minImpactSpeed)
            %data.damageObject(%vehicleObject, 0, VectorAdd(%vec, %vehicleObject.getPosition()),
                               ((%vecLen * %data.speedDamageScale) / 2), $DamageType::Ground);
      }
//      if(%vecLen > %data.minImpactSpeed)
//         %data.damageObject(%vehicleObject, 0, VectorAdd(%vec, %vehicleObject.getPosition()),
//                            %vecLen * 0.00997, $DamageType::Ground);
   }

   // associated "crash" sounds
   if(%vecLen > %vDataBlock.hardImpactSpeed)
      %vehicleObject.playAudio(0, %vDataBlock.hardImpactSound);
   else if(%vecLen > %vDataBlock.softImpactSpeed)
      %vehicleObject.playAudio(0, %vDataBlock.softImpactSound);
}

//------------------------------------------------------------------------------
// Water Physics
//------------------------------------------------------------------------------

function checkWaterPhysics(%obj)
{
   if(isObject(%obj))
   {
      if(%obj.isWet)
      {
         %mass = %obj.getMass();
         %iVec = vectorScale(%obj.getVelocity(), %mass);
         %obj.applyImpulse(%obj.getPosition(), %iVec);
   
         // speed tolerance checking (in KPH)
         %mass = %obj.getMass();
         %pct = %obj.getDamagePct() / 4;
         %max = %pct * (%data.minDrag * 10);
         %maxSpeed = %max - (%mass / 10);
         %vel = msToKPH(velToSingle(%obj.getVelocity()));
   
         if(%maxSpeed > %vel)
            %obj.getDatablock().damageObject(%obj, 0, "0 0 0", 0.01*(%maxSpeed - %vel), $DamageType::Default);
   
         schedule(100, 0, "checkWaterPhysics", %obj);
      }
   }
}

function VehicleData::onEnterLiquid(%data, %obj, %coverage, %type)
{
//   if(!%data.disableWaterPhysics && truPhysics.enabled)
//   {
      %obj.isWet = true;
//      %obj.wetObject = findWater(%obj.getPosition(), 0.1);
//      checkWaterPhysics(%obj);
//   }

   switch(%type)
   {
      case 0:
         //Water
         %obj.setHeat(0.0);
      case 1:
         //Ocean Water
         %obj.setHeat(0.0);
      case 2:
         //River Water
         %obj.setHeat(0.0);
      case 3:
         //Stagnant Water
         %obj.setHeat(0.0);
      case 4:
         //Lava
         %obj.liquidDamage(%data, $VehicleDamageLava, $DamageType::Lava);
      case 5:
         //Hot Lava
         %obj.liquidDamage(%data, $VehicleDamageHotLava, $DamageType::Lava);
      case 6:
         //Crusty Lava
         %obj.liquidDamage(%data, $VehicleDamageCrustyLava, $DamageType::Lava);
      case 7:
         //Quick Sand
   }
}

function VehicleData::onLeaveLiquid(%data, %obj, %type)
{
//   if(!%data.disableWaterPhysics && truPhysics.enabled)
//   {
      %obj.isWet = false;
//      %obj.wetObject = "";
//   }

   switch(%type)
   {
      case 0:
         //Water
         %obj.setHeat(1.0);
      case 1:
         //Ocean Water
         %obj.setHeat(1.0);
      case 2:
         //River Water
         %obj.setHeat(1.0);
      case 3:
         //Stagnant Water
         %obj.setHeat(1.0);
      case 4:
         //Lava
      case 5:
         //Hot Lava
      case 6:
         //Crusty Lava
      case 7:
         //Quick Sand
   }

   if(%obj.lDamageSchedule !$= "")
   {
      cancel(%obj.lDamageSchedule);
      %obj.lDamageSchedule = "";
   }
}
