if(isObject(Meltdown))
   Meltdown.DSOFileCount++;
else
   quit();

//--------------------------------------
// Tether Missile Launcher
//--------------------------------------

datablock SeekerProjectileData(ShoulderMissileT)
{
   casingShapeName     = "weapon_missile_casement.dts";
   projectileShapeName = "weapon_missile_projectile.dts";
   hasDamageRadius     = true;
   indirectDamage      = 0.75;
   damageRadius        = 12.0;
   radiusDamageType    = $DamageType::Missile;
   kickBackStrength    = 2000;

   explosion           = "LargeMissileExplosion";
   splash              = MissileSplash;
   velInheritFactor    = 1.0;    // to compensate for slow starting velocity, this value
                                 // is cranked up to full so the missile doesn't start
                                 // out behind the player when the player is moving
                                 // very quickly - bramage

   baseEmitter         = TMStreakEmitter;
   delayEmitter        = MissileFireEmitter;
   puffEmitter         = MissilePuffEmitter;
   bubbleEmitter       = GrenadeBubbleEmitter;
   bubbleEmitTime      = 1.0;

   exhaustEmitter      = MissileLauncherExhaustEmitter;
   exhaustTimeMs       = 300;
   exhaustNodeName     = "muzzlePoint1";

   lifetimeMS          = 15000;
   muzzleVelocity      = 10.0;
   maxVelocity         = 100.0;
   turningSpeed        = 1000.0;
   acceleration        = 10.0;

   proximityRadius     = 6;

   terrainAvoidanceSpeed         = 1000;
   terrainScanAhead              = 30;
   terrainHeightFail             = 15;
   terrainAvoidanceRadius        = 100;

   flareDistance = 200;
   flareAngle    = 30;

   sound = MissileProjectileSound;

   hasLight    = true;
   lightRadius = 5.0;
   lightColor  = "0.2 0.05 0";

   useFlechette = true;
   flechetteDelayMs = 550;
   casingDeb = FlechetteDebris;

   explodeOnWaterImpact = false;
};

//--------------------------------------------------------------------------
// Ammo
//--------------------------------------

datablock ItemData(TetherLauncherAmmo)
{
   className = Ammo;
   catagory = "Ammo";
   shapeFile = "ammo_missile.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   pickupRadius = 2;
	pickUpName = "some tether missiles";
};

//--------------------------------------------------------------------------
// Weapon
//--------------------------------------
datablock ItemData(TetherLauncher)
{
   className = Weapon;
   catagory = "Spawn Items";
   shapeFile = "weapon_missile.dts";
   image = TetherLauncherImage;
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   pickupRadius = 2;
	pickUpName = "a tether missile launcher";
   emap = true;
};

datablock ShapeBaseImageData(TetherLauncherImage)
{
   className = WeaponImage;
   shapeFile = "weapon_missile.dts";
   item = TetherLauncher;
   ammo = TetherLauncherAmmo;
   offset = "0 0 0";
   armThread = lookms;
   emap = true;

   stateName[0]                     = "Activate";
   stateTransitionOnTimeout[0]      = "ActivateReady";
   stateTimeoutValue[0]             = 0.5;
//   stateScript[0]                   = "onActivate";
   stateSequence[0]                 = "Activate";
   stateSound[0]                    = MissileSwitchSound;

   stateName[1]                     = "ActivateReady";
   stateTransitionOnLoaded[1]       = "Ready";
   stateTransitionOnNoAmmo[1]       = "NoAmmo";

   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "CheckWet";

   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.2;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";
   stateSound[3]                    = MissileFireSound;

   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 2.0;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateSound[4]                    = MissileReloadSound;

   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "NoAmmo";
   stateTransitionOnTriggerDown[5]  = "DryFire";

   stateName[6]                     = "DryFire";
   stateSound[6]                    = MissileDryFireSound;
   stateTimeoutValue[6]             = 1.0;
   stateTransitionOnTimeout[6]      = "ActivateReady";
   
   stateName[7]                     = "CheckWet";
   stateTransitionOnWet[7]          = "WetFire";
   stateTransitionOnNotWet[7]       = "Fire";
   
   stateName[8]                     = "WetFire";
   stateTransitionOnNoAmmo[8]       = "NoAmmo";
   stateTransitionOnTimeout[8]      = "Reload";
   stateSound[8]                    = MissileDryFireSound;
   stateTimeoutValue[8]             = 0.5;
};

datablock ShapeBaseImageData(RocketTLImage)
{
   className = WeaponImage;
   shapeFile = "weapon_plasma.dts";
   emap = true;
   offset = "0 0 0.25";
   rotation = "0 1 0 180";
   armThread = lookms;
   emap = true;
   item = TetherLauncher;

   stateName[0]                     = "Activate";
   stateSequence[0]                 = "Activate";
};

$ScanRange = 325;

function TetherPositionLoop(%obj, %slot)
{
     if(isObject(%obj) && %obj.getMountedImage(%slot).item $= "TetherLauncher")
     {
        %mVec = %obj.getMuzzleVector(0);
        %mPos = %obj.getMuzzlePoint(0);
        %nmVec = VectorNormalize(%mVec);
        %scmVec = VectorScale(%nmVec, $ScanRange);
        %mEnd = VectorAdd(%mPos, %scmVec);
        %searchResult = containerRayCast(%mPos, %mEnd, $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType | $TypeMasks::StaticShapeObjectType | $TypeMasks::TurretObjectType | $TypeMasks::ItemObjectType, 0);
        %raycastPt = posFrRaycast(%searchResult);

         if(!%searchresult)
         {
            %pos = %obj.getMuzzlePoint(0);
            %nvec = vectorNormalize(%obj.getMuzzleVector(0));
            %vec = vectorScale(%nvec, $ScanRange);
            %nPos = vectorAdd(%pos, %vec);
            %obj.tetherBeacon.setPosition(%nPos);
         }
         else
            %obj.tetherBeacon.setPosition(%raycastPt);

         schedule(32, 0, "TetherPositionLoop", %obj, %slot);
     }
     else
     {
          %obj.tetherBeacon.delete();
          %obj.tetherBeacon = "";

          if(isObject(%obj.t))
             %obj.t.delete();
     }
}

function TetherLauncherImage::onFire(%data,%obj,%slot)
{
   %data.lightStart = getSimTime();
         %weapon = %obj.getMountedImage(0).item;

      %projectile = "ShoulderMissileT";

      %p = new SeekerProjectile() {
         dataBlock        = %projectile;
         initialDirection = %obj.getMuzzleVector(%slot);
         initialPosition  = %obj.getMuzzlePoint(%slot);
         sourceObject     = %obj;
         sourceSlot       = 4;
         vehicleObject    = 0;
      };

   if (isObject(%obj.lastProjectile) && %obj.deleteLastProjectile)
      %obj.lastProjectile.delete();

   %obj.lastProjectile = %p;
   %obj.deleteLastProjectile = %data.deleteLastProjectile;
   MissionCleanup.add(%p);
//   MissileSet.add(%p);
   %p.setObjectTarget(%obj.tetherBeacon);

   // AI hook
   if(%obj.client)
      %obj.client.projectile = %p;

    %obj.decInventory(%data.ammo,1);
}

datablock ShapeBaseImageData(RktDecalAImage)
{
   shapeFile = "weapon_disc.dts";
   offset = "0.125 0.1 0.125";
   rotation = "0 1 0 90";
   emap = true;
};

datablock TargetProjectileData(tBasicTargeter)
{
   directDamage        	= 0.0;
   hasDamageRadius     	= false;
   indirectDamage      	= 0.0;
   damageRadius        	= 0.0;
   velInheritFactor    	= 1.0;

   maxRifleRange       	= 325;
   beamColor           	= "0.1 1.0 0.1";

   startBeamWidth			= 0.20;
   pulseBeamWidth 	   = 0.15;
   beamFlareAngle 	   = 3.0;
   minFlareSize        	= 0.0;
   maxFlareSize        	= 400.0;
   pulseSpeed          	= 6.0;
   pulseLength         	= 0.150;

   textureName[0]      	= "special/nonlingradient";
   textureName[1]      	= "special/flare";
   textureName[2]      	= "special/pulse";
   textureName[3]      	= "special/expFlare";
   beacon               = true;
};

function TetherLauncherImage::onMount(%data,%obj,%slot)
{
     Parent::onMount(%data,%obj,%slot);
     %mVec = %obj.getMuzzleVector(0);
     %mPos = %obj.getMuzzlePoint(0);
     %nmVec = VectorNormalize(%mVec);
     %scmVec = VectorScale(%nmVec, $ScanRange);
     %mEnd = VectorAdd(%mPos, %scmVec);
     %searchResult = containerRayCast(%mPos, %mEnd, $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType | $TypeMasks::StaticShapeObjectType | $TypeMasks::TurretObjectType | $TypeMasks::ItemObjectType, 0);
     %raycastPt = posFrRaycast(%searchResult);

     if(%obj.tetherBeacon)
     {
          %obj.tetherBeacon.delete();
          %obj.tetherBeacon = "";
     }

     %obj.tetherBeacon = new WayPoint()
     {
          position = %raycastPt;
          rotation = "1 0 0 0";
          scale = "1 1 1";
          name = "";
          dataBlock = "WayPointMarker";
          lockCount = "0";
          homingCount = "0";
          team = %obj.client.team;
       };
//       %obj.tetherBeacon = new ScopeAlwaysShape()
//       {
//          dataBlock = "BomberBeacon";
//          position = %raycastPt;
//          name = "Target";
//       };

//       %obj.tetherBeacon.playThread($AmbientThread, "ambient");
//       %obj.tetherBeacon.team = %obj.team;
//       %obj.tetherBeacon.sourceObject = %obj;
//       %obj.tetherBeacon.setTarget(%obj.team);
//
       MissionCleanup.add(%obj.tetherBeacon);

      %t = new TargetProjectile() {
         dataBlock        = "tBasicTargeter";
         initialDirection = %obj.getMuzzleVector(%slot);
         initialPosition  = %obj.getMuzzlePoint(%slot);
         sourceObject     = %obj;
         sourceSlot       = 0;
         vehicleObject    = 0;
      };
      MissionCleanup.add(%t);
      %obj.t = %t;

     TetherPositionLoop(%obj, %slot);

   %obj.mountImage(RocketTLImage, 4);
   %obj.mountImage(RktDecalAImage, 5);
}

function TetherLauncherImage::onUnmount(%this,%obj,%slot)
{
   if(isObject(%obj.t))
      %obj.t.delete();

   Parent::onUnmount(%this, %obj, %slot);
   %obj.unmountImage(4);
   %obj.unmountImage(5);
}
