
// --------------------------------------------
// Defender Designator

$TeamDeployableMax[DefenderDesignator] = 3;
$TeamDeployableMin[DefenderDesignator] = 3;

datablock ShapeBaseImageData(DefenderDesignatorImage)
{
   mass = 15;

   shapeFile = "pack_deploy_turreti.dts";
   item = DefenderDesignator;
   mountPoint = 1;
   offset = "0 0 0";

   stateName[0] = "Idle";
   stateTransitionOnTriggerDown[0] = "Activate";

   stateName[1] = "Activate";
   stateScript[1] = "onActivate";
   stateTransitionOnTriggerUp[1] = "Idle";

   emap = true;

   maxDepSlope = 360;
   deploySound = TurretDeploySound;

   minDeployDis                       =  0.5;
   maxDeployDis                       =  5.0;  //meters from body
};

datablock ItemData(DefenderDesignator)
{
   className = Pack;
   catagory = "Deployables";
   shapeFile = "pack_deploy_turreti.dts";
   mass = 3.0;
   elasticity = 0.2;
   friction = 0.6;
   pickupRadius = 1;
   rotate = false;
   image = "DefenderDesignatorImage";
   pickUpName = "a defender designator";

   emap = true;
};

function DefenderDesignatorImage::onDeploy(%item, %plyr, %slot)
{
   if(%item.item $= "MotionSensorDeployable" || %item.item $= "PulseSensorDeployable")
   {
      %plyr.deploySensors--;
      %plyr.client.updateSensorPackText(%plyr.deploySensors);
      if(%plyr.deploySensors <= 0)
      {
         // take the deployable off the player's back and out of inventory
         %plyr.unmountImage(%slot);
         %plyr.decInventory(%item.item, 1);
      }
   }
   else
   {
      // take the deployable off the player's back and out of inventory
      %plyr.unmountImage(%slot);
      %plyr.decInventory(%item.item, 1);
   }

   // create the actual deployable
   %rot = %item.getInitialRotation(%plyr);
   if(%item.deployed.className $= "DeployedTurret")
      %className = "Turret";
   else
      %className = "StaticShape";

   %deplObj = new (%className)() {
      dataBlock = %item.deployed;
   };


   // set orientation
   if(%className $= "Turret")
      %deplObj.setDeployRotation(%item.surfacePt, %item.surfaceNrm);
   else
      %deplObj.setTransform(%item.surfacePt SPC %rot);

   // set the recharge rate right away
   if(%deplObj.getDatablock().rechargeRate)
      %deplObj.setRechargeRate(%deplObj.getDatablock().rechargeRate);

   // set team, owner, and handle
   %deplObj.team = %plyr.client.Team;
   %deplObj.owner = %plyr.client;

   // set the sensor group if it needs one
   if(%deplObj.getTarget() != -1)
      setTargetSensorGroup(%deplObj.getTarget(), %plyr.client.team);

   // place the deployable in the MissionCleanup/Deployables group (AI reasons)
   addToDeployGroup(%deplObj);

   //let the AI know as well...
   AIDeployObject(%plyr.client, %deplObj);

   // play the deploy sound
   serverPlay3D(%item.deploySound, %deplObj.getTransform());

   // increment the team count for this deployed object

   $TeamDeployedCount[%plyr.team, "DefenderDesignator"]++;
   %deplObj.deploy();

   detectDeployingOn(%deplObj, %item.surfacePt);
}

function DefenderDesignatorImage::getInitialRotation(%item, %plyr)
{
   %surfaceAngle = getTerrainAngle(%item.surfaceNrm);
   if(%surfaceAngle > 155)
      %item.deployed = DefenderDesignatorCeilingIndoor;
   else if(%surfaceAngle > 45)
      %item.deployed = DefenderDesignatorWallIndoor;
   else
      %item.deployed = DefenderDesignatorFloorIndoor;
}

function DefenderDesignatorImage::onPickup(%this, %obj, %shape, %amount)
{
   // created to prevent console errors
}

function DefenderDesignatorWallIndoor::onDestroyed(%this, %obj, %prevState)
{
     Parent::onDestroyed(%this, %obj, %prevState);
     $TeamDeployedCount[%obj.team, "DefenderDesignator"]--;
     %obj.schedule(750, "delete");
}

function DefenderDesignatorFloorIndoor::onDestroyed(%this, %obj, %prevState)
{
     Parent::onDestroyed(%this, %obj, %prevState);
     $TeamDeployedCount[%obj.team, "DefenderDesignator"]--;
     %obj.schedule(750, "delete");
}

function DefenderDesignatorCeilingIndoor::onDestroyed(%this, %obj, %prevState)
{
     Parent::onDestroyed(%this, %obj, %prevState);
     $TeamDeployedCount[%obj.team, "DefenderDesignator"]--;
     %obj.schedule(750, "delete");
}

//====================================== Laser Sentry Turret
$TeamDeployableMax[LaserSentryPack] = 2;

datablock TurretData(LaserSentryTurret) : TurretDamageProfile
{
   catagory = "Turrets";
   shapeFile = "turret_sentry.dts";
   mass = 5.0;

   barrel = LaserSentryBarrel;

   maxDamage = 0.5;
   destroyedLevel = 0.5;
   disabledLevel = 0.433;
   explosion      = ShapeExplosion;
   expDmgRadius = 5.0;
   expDamage = 0.4;
   expImpulse = 1000.0;
   repairRate = 0;

   thetaMin = 89;
   thetaMax = 175;
   emap = true;

   isShielded           = true;
   energyPerDamagePoint = 100;
   maxEnergy = 125;
   rechargeRate = 0.40;

   canControl = true;
   cmdCategory = "Tactical";
   cmdIcon = CMDTurretIcon;
   cmdMiniIconName = "commander/MiniIcons/com_turret_grey";
   targetNameTag = 'Laser Sentry';
   targetTypeTag = 'Turret';
   sensorData = SentryMotionSensor;
   sensorRadius = SentryMotionSensor.detectRadius;
   sensorColor = "9 136 255";
   firstPersonOnly = true;
};

datablock TurretImageData(LaserSentryBarrel)
{
   shapeFile = "turret_muzzlepoint.dts";

   projectile = BasicSniperShot;
   projectileType = SniperProjectile;
   usesEnergy = true;
   fireEnergy = 10.00;
   minEnergy = 10.00;
   emap = true;

   // Turret parameters
   activationMS      = 300;
   deactivateDelayMS = 500;
   thinkTimeMS       = 200;
   degPerSecTheta    = 520;
   degPerSecPhi      = 960;
   attackRadius      = 50;

   // State transitions
   stateName[0]                     = "Activate";
   stateTimeoutValue[0]             = 0.01;
   stateTransitionOnTimeout[0]      = "Ready";
   stateSound[0]                    = SentryTurretSwitchSound;

   stateName[1]                     = "Ready";
   stateTransitionOnTriggerDown[1]  = "Fire";
   stateTransitionOnNoAmmo[1]       = "NoAmmo";

   stateName[2]                     = "Fire";
   stateTransitionOnTimeout[2]      = "Reload";
   stateTimeoutValue[2]             = 0.25;
   stateFire[2]                     = true;
   stateRecoil[2]                   = LightRecoil;
   stateAllowImageChange[2]         = false;
   stateSequence[2]                 = "Fire";
   stateSound[2]                    = "SniperRifleFireSound"; //SentryTurretFireSound;
   stateScript[2]                   = "onFire";

   stateName[3]                     = "Reload";
   stateTimeoutValue[3]             = 0.5;
   stateAllowImageChange[3]         = false;
   stateTransitionOnTimeout[3]      = "Ready";
   stateTransitionOnNoAmmo[3]       = "NoAmmo";

   stateName[4]                     = "NoAmmo";
   stateTransitionOnAmmo[4]         = "Reload";
   stateSequence[4]                 = "NoAmmo";
};

function xLaserSentryBarrel::onFire(%data,%obj,%slot)
{
   if(%obj.getEnergyLevel() < 10)
      return;

   %pct = 1; //%obj.getEnergyLevel() / %obj.getDataBlock().maxEnergy;
   %p = new SniperProjectile()
   {
      dataBlock        = BasicSniperShot;
      initialDirection = %obj.getMuzzleVector(%slot);
      initialPosition  = %obj.getMuzzlePoint(%slot);
      sourceObject     = %obj;
      damageFactor     = 0.0;
      sourceSlot       = %slot;
   };
   MissionCleanup.add(%p);

   %p.setEnergyPercentage(%pct);
//   %p.doNormalDamage = true;
//   %p.normalDmg = 0;

//   serverPlay3D(SniperRifleFireSound, %obj.getTransform());

   %obj.useEnergy(10);
}

datablock ShapeBaseImageData(LaserSentryPackImage)
{
   mass = 15;
   shapeFile = "pack_deploy_turreti.dts";
   item = LaserSentryPack;
   mountPoint = 1;
   offset = "0 0 0";
   deployed = "LaserSentryTurret";

   stateName[0] = "Idle";
   stateTransitionOnTriggerDown[0] = "Activate";

   stateName[1] = "Activate";
   stateScript[1] = "onActivate";
   stateTransitionOnTriggerUp[1] = "Idle";

   emap = true;

   maxDepSlope = 360;
   deploySound = TurretDeploySound;

   minDeployDis                       =  0.5;
   maxDeployDis                       =  5.0;  //meters from body
};

datablock ItemData(LaserSentryPack)
{
   className = Pack;
   catagory = "Deployables";
   shapeFile = "pack_deploy_turreti.dts";
   mass = 3.0;
   elasticity = 0.2;
   friction = 0.6;
   pickupRadius = 1;
   rotate = false;
   image = "LaserSentryPackImage";
   pickUpName = "a deployable laser sentry";
   emap = true;
};

function LaserSentryPackImage::testNoTerrainFound(%item)
{
}

function LaserSentryPackImage::testNoInteriorFound(%item)
{
   return %item.surface.getClassName() !$= InteriorInstance;
}

function LaserSentryPackImage::testObjectTooClose(%item)
{
//   return false;

   %mask =    ($TypeMasks::VehicleObjectType     | $TypeMasks::MoveableObjectType   |
               $TypeMasks::ForceFieldObjectType  | $TypeMasks::ItemObjectType       |
               $TypeMasks::PlayerObjectType      | $TypeMasks::TurretObjectType);

   InitContainerRadiusSearch( %item.surfacePt, 0.5, %mask );

   %test = containerSearchNext();
   return %test;
}

function LaserSentryTurret::checkDeployPos(%item)
{
   return true;
}

function LaserSentryPackImage::onDeploy(%item, %plyr, %slot)
{
   %plyr.unmountImage(%slot);
   %plyr.decInventory(%item.item, 1);

   %rot = %item.getInitialRotation(%plyr);

   %deplObj = new Turret()
   {
      dataBlock = %item.deployed;
      initialBarrel = "LaserSentryBarrel";
   };

   %deplObj.setDeployRotation(%item.surfacePt, vectorNeg(%item.surfaceNrm));
//   %deplObj.setTransform(%item.surfacePt SPC %rot);

   %deplObj.setSelfPowered();
   if(%deplObj.getDatablock().rechargeRate)
      %deplObj.setRechargeRate(%deplObj.getDatablock().rechargeRate);

   %deplObj.team = %plyr.client.Team;
   %deplObj.owner = %plyr.client;

   if(%deplObj.getTarget() != -1)
      setTargetSensorGroup(%deplObj.getTarget(), %plyr.client.team);

   addToDeployGroup(%deplObj);
   AIDeployObject(%plyr.client, %deplObj);
   serverPlay3D(%item.deploySound, %deplObj.getTransform());
   $TeamDeployedCount[%plyr.team, "LaserSentryPack"]++;
   %deplObj.deploy();

   detectDeployingOn(%beacon, %item.surfacePt);
}

function LaserSentryPackImage::onPickup(%this, %obj, %shape, %amount)
{
   // created to prevent console errors
}

function LaserSentryTurret::onDestroyed(%this, %obj, %prevState)
{
   Parent::onDestroyed(%this, %obj, %prevState);
   $TeamDeployedCount[%obj.team, "LaserSentryPack"]--;
   %obj.schedule(750, "delete");
}
